package main

import "core:math"
import "core:math/rand"
import rl "vendor:raylib"

Vec3 :: [3]f32

Entity :: struct {
	position: Vec3,
	velocity: Vec3,
	size:     Vec3,
}

PAL_INT :: [6]u32{0x272744ff, 0x494d7eff, 0x8b6d9cff, 0xc69fa5ff, 0xf2d3abff, 0xfbf5efff}

PAL := [6]rl.Color{}

CAMERA: ^rl.Camera3D
TEXTURES: Textures

discard_shader: rl.Shader // See shader below. Required for drawing sprites

main :: proc() {
	for v, i in PAL_INT {
		PAL[i] = rl.GetColor(v)
	}

	rl.InitWindow(800, 400, "By The Fire")
	rl.SetTargetFPS(60)

	TEXTURES = load_textures()
	defer unload_textures(&TEXTURES)

	discard_shader = rl.LoadShaderFromMemory(nil, alpha_discard)

	// Floor checkerboard
	img := rl.GenImageChecked(64, 64, 8, 8, PAL[1], PAL[2])
	defer rl.UnloadImage(img)
	tex := rl.LoadTextureFromImage(img)
	defer rl.UnloadTexture(tex)

	// Floor mesh
	plane := rl.GenMeshPlane(32, 32, 16, 16)
	defer rl.UnloadMesh(plane)
	mat := rl.LoadMaterialDefault()
	rl.SetMaterialTexture(&mat, rl.MaterialMapIndex.ALBEDO, tex)

	player_load_resources()
	defer player_unload_resources()
	player := player_create()

	CAMERA = &(player.camera)

	torches := [10]Torch{}
	for &v, i in torches {
		torches[i] = Torch {
			position = Vec3{rand.float32_range(-10, 10), 2, rand.float32_range(-10, 10)},
		}
	}

	rl.DisableCursor()

	for !rl.WindowShouldClose() {
		using rl
		player_update(&player, GetFrameTime())

		BeginDrawing()
		BeginMode3D(player.camera)
		BeginShaderMode(discard_shader)

		ClearBackground(PAL[0])
		rlPushMatrix()
		rlRotatef(45, 0, 1, 0)
		DrawMesh(plane, mat, rl.Matrix(1))
		rlPopMatrix()

		player_draw(&player)
		for &v in torches {
			torch_draw(&v)
		}

		EndShaderMode()

		EndMode3D()
		EndDrawing()
	}
}


alpha_discard :: `
#version 330

// Input vertex attributes (from vertex shader)
in vec2 fragTexCoord;
in vec4 fragColor;

// Input uniform values
uniform sampler2D texture0;
uniform vec4 colDiffuse;

// Output fragment color
out vec4 finalColor;

void main()
{
    vec4 texelColor = texture(texture0, fragTexCoord);
    if (texelColor.a == 0.0) discard;
    finalColor = texelColor * fragColor * colDiffuse;
}
`
